#!/bin/bash

# This awk script reads in the workspace default.qubo file built
# by "dw embed" and converts it into an awk program which we can run
# to generate qbsolv input.

awk '

BEGIN {
    print "BEGIN {"
    print "    set_params()"
    print "    print_header()"
    linear_terms = 0;
    quadratic_terms = 0;
}

$1 ~ /^VARIABLE/ {
    sub(/^VARIABLE\("/,"",$1);
    sub(/"\):/,"",$1);
    print "    " $1, "= 1";
    var_id = $2;
    sub(/^id=/,"",var_id);
    if (0 + max_var_id < 0 + var_id) max_var_id = var_id;
}

$1 ~ /^TERM/ {
    sub(/^TERM\(/,"",$1);
    sub(/\):/,"",$1);
    ind = $1;
    $1 = "";
    degree = split(ind, ind_arr, ",");
    sub(/expr="/,"");
    sub(/"$/,"");
    if (degree == 1)
    {
        linear_terms = 1 + linear_terms;
        line_lin[linear_terms] = sprintf("    print %d, %d,%s", ind_arr[1], ind_arr[1], $0);
    }
    else if (degree == 2)
    {
        quadratic_terms = 1 + quadratic_terms;
        line_quad[quadratic_terms] = sprintf("    print %d, %d,%s", ind_arr[1], ind_arr[2], $0);
    }
}

END {
    for (i=1; i<=   linear_terms; ++i) print line_lin[i];
    for (i=1; i<=quadratic_terms; ++i) print line_quad[i];

    print "}"
    print ""

    max_var_id = 1 + max_var_id

    print "function print_header()"
    print "{"
    print "    print \"p qubo 0 \" " max_var_id ",", linear_terms ",", quadratic_terms
    print "}"

    printf("\nQBSOLV INPUT:\n\tvariables=%d\n", max_var_id) > "/dev/stderr"
    printf("\tlinear terms=%d\n", linear_terms) > "/dev/stderr"
    printf("\tquadratic terms=%d\n\n", quadratic_terms) > "/dev/stderr"
}

'
