#!/bin/bash

shopt -s extglob

if [[ -z $1 || -z $2 || $2 != mode=@(embed|qbsolv) ]]
then
    echo "usage: dw-test.bash N mode={embed|qbsolv}"
    exit 1
fi

test_num=$1
test_mode=$2

# The following dw commands are common to all tests:

dw set connection laptop > /dev/null
dw set solver c4-sw_sample > /dev/null
dw mkdir -p compiler
dw cd compiler
dw init

mode=${test_mode#mode=}

case ${mode} in
    (embed)
	mode_embed=1
	command=embed
	input_suffix=qmi
	output_suffix=sol
	;;
    (qbsolv)
	mode_qbsolv=1
	command=qbsolv
	input_suffix=qbi
	output_suffix=qbo
	;;
    (*)
	printf "\nmode ${mode} not supported\n\n"
	exit 1;;
esac

case ${test_num} in
    (0)
        dw set param p
	dw set variable q0
	dw set variable q1
	dw set variable q2
	dw set assert 'q0 + q1 - q2'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=2}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 3 ; TOTAL VALID in range [980, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (1)
        dw set param s
        dw set param t
	dw set variable q0
	dw set variable q1
	dw set variable q2
	dw set param s
	dw set assert 'q0 + q1 - q2'
	dw set param t
	dw set assert 'q0 + q1 - 1'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind s=1 t=1 ${mode_embed+param_chain=2}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 2 ; TOTAL VALID in range [650, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (2)
        dw set param p
	dw set variable i0
	dw set variable i1
	dw set variable o0
	dw set variable o1
	dw set variable s
	dw set assert '( (s==0 && o0==i0 && o1==i1) || (s==1 && o1==i0 && o0==i1) ) - 1'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 param_prod=2 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 8 ; TOTAL VALID in range [800, 950]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (3)
        dw set param p
	dw set variable q0
	dw set variable q1
	dw set variable q2
	dw set variable q3
	dw set assert '(q0 + 2*q1) * (q2 + 2*q3) - 6'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 2 ; TOTAL VALID in range [990, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (4)
        dw set param s
        dw set param t
	dw set variable q0
	dw set variable q1
	dw set variable q2
	dw set variable q3
	dw set variable q4
	dw set variable q5
	dw set param s
	dw set assert '(q0 + q1) - q2'
	dw set assert '(q1 + q3) - q5'
	dw set param t
	dw set assert '(q2 * q4) - q0'
	dw set assert '((q0 + 2*q1) * (q2 + 2*q3)) - 6'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind s=1 t=1 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 0"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS NOT VALID"
	;;
    (5)
        dw set param p
	dw set variable r
	dw set variable g
	dw set variable b
	dw set variable y
	dw set assert '(r + g + b + y) - 1'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=2}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 4; TOTAL VALID in range [990, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (6)
        dw set param p
	dw set variable i0
	dw set variable i1
	dw set variable i2
	dw set variable o0
	dw set variable o1
	dw set assert '(i0 + i1 + i2) - (2 * o1 + o0)'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 8; TOTAL VALID in range [900, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (7)
        dw set param p
	dw set variable a
	dw set variable b
	dw set variable c
	dw set assert 'a + b + c > 0'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 1; TOTAL VALID in range [1000, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (8)
        dw set param p
	dw set variable a
	dw set variable b
	dw set variable c
	dw set assert 'a + b + c == 0'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 param_prod=2 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 7; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (9)
        dw set param p
	dw set variable x
	dw set variable y
	dw set variable z
	dw set assert 'x*y - z'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 4; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (10)
        dw set param p
	dw set variable r
	dw set variable g
	dw set variable b
	dw set assert '(r + g + b) - 1'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=2}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 3; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (11)
        dw set param p
	dw set variable x
	dw set variable y
	dw set variable z
	dw set assert '(1-x)*(1-y) - (1-z)'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=4}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 4; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (12)
        dw set param p
	dw set variable x
	dw set variable y
	dw set assert '(x-y)**2'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 2; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (13)
        dw set param p
	dw set variable x
	dw set variable y
	dw set assert '(1-x)*y'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 3; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (14)
        dw set param p
	dw set variable r1
	dw set variable r2
	dw set variable g1
	dw set variable g2
	dw set assert 'r1*r2 + g1*g2'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 9; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (15)
        dw set param p
	dw set variable x
	dw set variable y
	dw set variable z
	dw set assert '(2*(x+y+z) - 3)**2 - 1'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1 ${mode_embed+param_chain=5}
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 6; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (16)
        dw set param p
	dw set variable x
	dw set variable y
	dw set variable z
	dw set assert 'x - y'
	dw compile > test-${test_num}.q
	dw ${command} test-${test_num}.q
	dw bind p=1
	dw exec num_reads=1000 default.${input_suffix}
	dw val default.${output_suffix}
	[[ -n ${mode_embed} ]] && echo "EXPECTED VALID = 4; TOTAL VALID in range [950, 1000]"
	[[ -n ${mode_qbsolv} ]] && echo "EXPECTED SOLUTION IS VALID"
	;;
    (*)
	printf "\nTest numbers 0 thru 16 only are supported\n\n"
	exit 1;;
esac
