#!/bin/bash

# Create the most general three-variable QUBO using parameters.
# Set the parameters to cause the QUBO to emulate an AND gate and run it.
# Set the parameters to cause the QUBO to emulate an OR gate and run it.

# Exit if not being run via Bash shell:
if [ ! -n "$BASH" ] ; then 
    echo Please run this script $0 with bash; exit 1; 
fi

# Get a connection to the local simulator:
dw set connection laptop

# Get a 128-qubit sampling solver from the simulator:
dw set solver c4-sw_sample

# Make new workspace directory (examples) if it does not exist:
dw mkdir -p examples

# Change to new workspace directory:
dw cd examples

# Initialize the workspace:
dw init

# Create a three variable QUBO which is completely parameterized:
echo "
param: a1
param: b1
param: c1
param: d2
param: e2
param: f2

var: x
var: y
var: z

term: a1 * x
term: b1 * y
term: c1 * z

term: d2 * x * y
term: e2 * x * z
term: f2 * y * z
" > E-qubo.q

# Embed the QUBO:
dw embed E-qubo.q

# Create a bind file to implement z=AND(x,y) and generate the QMI:
echo "
a1 = 0
b1 = 0
c1 = 3
d2 = 1
e2 = -2
f2 = -2
" > E-qubo-and.b
dw bind param_chain=5 E-qubo-and.b

# Execute the QMI:
dw exec num_reads=1000 E-qubo-and.qmi

# Display the sample statistics:
dw val -v E-qubo-and.sol

# Display the first four solutions:
for ((s=1; s<=4; ++s))
do
    dw val -v -s $s E-qubo-and.sol
done

# Create a bind file to implement z=OR(x,y) and generate the QMI:
echo "
a1 = 1
b1 = 1
c1 = 1
d2 = 1
e2 = -2
f2 = -2
" > E-qubo-or.b
dw bind param_chain=5 E-qubo-or.b

# Execute the QMI:
dw exec num_reads=1000 E-qubo-or.qmi

# Display the sample statistics:
dw val -v E-qubo-or.sol

# Display the first four solutions:
for ((s=1; s<=4; ++s))
do
    dw val -v -s $s E-qubo-or.sol
done
